'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

# just the imports for the game
import data
import pygame
import sys
import math
import os
import time
import random
from pygame.locals import *

# Here are the constants for the game
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
BLUE = (0, 100, 255)
clock = pygame.time.Clock()

# this function loads an image from the data folder into pygame
def loadIm(name):
    file = data.filepath(name)
    try:
        image = pygame.image.load(file)
        return image.convert_alpha()
    except:
        name = 'error.png'
        print 'Could not load:', file
        file = data.filepath(name)
        image = pygame.image.load(file)
        return image.convert_alpha()




#Here is all the object classes for the game


class Octopus(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.location = 0
        self.images = [loadIm("OctopusAnimation/1.png"), loadIm("OctopusAnimation/2.png"), loadIm("OctopusAnimation/3.png"), loadIm("OctopusAnimation/4.png"), loadIm("OctopusAnimation/5.png")]
        self.image = self.images[self.location]
        self.type = "octopus"
        #set whether the player can move these directions
        self.up = True
        self.down = True
        self.left = True
        self.right = True
        self.rect = self.image.get_rect()
        self.x = x
        self.y = y
        self.fallspeed = 0
        self.rect.topleft = (self.x, self.y)
        
    def update(self, direction):
        self.location += direction/4.0
        self.location = self.location%4
        self.image = self.images[int(self.location)]
        self.y += self.fallspeed
        self.fallspeed += 0.1
        if self.x > 640 - self.rect.w:
            self.x = 640 - self.rect.w
        if self.x < 0:
            self.x = 0
        self.rect.topleft = (self.x, self.y)



class Collider(pygame.sprite.Sprite):
    def __init__(self, (x, y), (ox, oy), (w, h), direction):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = pygame.Surface((w, h), 0, 32)
        self.image.fill(BLUE)
        self.rect = Rect(x+ox, y+oy, w, h)
        self.ox, self.oy = ox, oy
        self.type = direction
        self.direction = direction
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)
        
    def update(self):
        self.rect.topleft = (self.x+self.ox, self.y+self.oy)

  

class Dirt(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = loadIm("DirtBlock.png")
        self.rect = self.image.get_rect()
        self.type = "dirt"
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)


class TP(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = loadIm("DirtBlock.png")
        self.rect = self.image.get_rect()
        self.type = "dirt"
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)



class Grass(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = loadIm("GrassBlock.png")
        self.rect = self.image.get_rect()
        self.type = "grass"
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)
        


class Sky(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = loadIm("Sky.png")
        self.rect = self.image.get_rect()
        self.type = "sky"
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)



class Water(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = loadIm("WaterSmall.png")
        self.type = "water"
        self.rect = self.image.get_rect()
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)



class Waves(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = loadIm("WavesSmall.png")
        self.type = "waves"
        self.rect = self.image.get_rect()
        self.x = x
        self.y = y
        self.rect.topleft = (self.x, self.y)




# controls the menu
def Menu(screen):
    background = loadIm("Menu.png")
    start = loadIm("Start.png")
    quit = loadIm("Quit.png")
    # draw the msnu
    screen.blit(background, (0,0))
    screen.blit(start, (100,220))
    screen.blit(quit, (100,290))
    # this monitors which menu item is selected
    selected = 0
    pygame.display.update()
    #when this is false the menu closes
    displayMenu = True
    while displayMenu:
        # loads the appropriate images
        if selected == 0:
            start = loadIm("StartSelected.png")
            quit = loadIm("Quit.png")
        elif selected == 1:
            start = loadIm("Start.png")
            quit = loadIm("QuitSelected.png")
        # display the updated menu items
        screen.blit(start, (100,220))
        screen.blit(quit, (100,290))
        pygame.display.update()
        for event in pygame.event.get():
            #if the close button is pressed
            if event.type == pygame.QUIT:
                sys.exit()
            if event.type == KEYDOWN:
                #if the escape key is pressed
                if event.key == K_ESCAPE:
                    sys.exit()
                #if the down button is pressed
                elif event.key == K_DOWN:
                    if selected == 0:
                        selected = 1
                #if the up button is pressed
                elif event.key == K_UP:
                    if selected == 1:
                        selected = 0
                #if the enter (return) button is pressed
                elif event.key == K_RETURN:
                    if selected == 0:
                        displayMenu = False
                    elif selected == 1:
                        sys.exit()



def LoadLevel(name, screen):
    # These are the sprite groups
    staticG = pygame.sprite.Group()
    octopusG = pygame.sprite.Group()
    backgroundG = pygame.sprite.Group()
    colliderG = pygame.sprite.Group()
    # This adds the Class to the group
    Dirt.containers = staticG
    Grass.containers = staticG
    Water.containers = staticG
    Waves.containers = staticG
    Sky.containers = backgroundG
    Collider.containers = colliderG
    Sky(0,0)
    Octopus.containers = octopusG
    file = data.load(name).readlines()
    for line in file:
        part = line.split(",")
        if part[0] == "dirt":
            Dirt(int(part[1]), int(part[2]))
        elif part[0] == "grass":
            Grass(int(part[1]), int(part[2]))
        elif part[0] == "water":
            Water(int(part[1]), int(part[2]))
        elif part[0] == "waves":
            Waves(int(part[1]), int(part[2]))
        elif part[0] == "octopus":
            Octopus(int(part[1]), int(part[2]))
            Collider((int(part[1]), int(part[2])), (0, 49), (64, 1), "down")
            Collider((int(part[1]), int(part[2])), (-1, 37), (1, 10), "left")
            Collider((int(part[1]), int(part[2])), (65, 37), (1, 10), "right")

    screen.fill(WHITE)
    backgroundG.draw(screen)
    staticG.draw(screen)
    octopusG.draw(screen)
    colliderG.draw(screen)
    pygame.display.update()
    return backgroundG, staticG, octopusG, colliderG










# the actual game code
def Game(screen):
    level = 2
    backgroundG, staticG, octopusG, colliderG = LoadLevel("Levels/" +str(level) + ".txt", screen)
    up = False
    down = False
    left = False
    right = False
    water = False
    jumped = False
    while True:
        #this handles all the events
        for event in pygame.event.get():
            if event.type == QUIT:
                sys.exit()
            if event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    Menu(screen)
                # here is all the keypresses for the movement
                if event.key == K_DOWN:
                    down = True
                elif event.key == K_UP:
                    up = True
                elif event.key == K_LEFT:
                    left = True
                elif event.key == K_RIGHT:
                    right = True
            elif event.type == KEYUP:
                if event.key == K_DOWN:
                    down = False
                elif event.key == K_UP:
                    up = False
                elif event.key == K_LEFT:
                    left = False
                elif event.key == K_RIGHT:
                    right = False
        for item in octopusG:
            water = False
            for staticitem in staticG:
                if item.rect.colliderect(staticitem.rect):
                    if staticitem.type == "water" or staticitem.type == "waves":
                        item.fallspeed /= 1.2
                        water = True
            direction = 0
            if down == True and item.down == True:
                item.y += 1.5
            if up == True:
                if item.down == False or water == True:
                    item.y -= 3
                    # to test if the character was jumping last frames
                    jumped = True
                elif jumped == True:
                    item.y -= 3
            else:
                if jumped == True:
                    item.fallspeed -= 3
                jumped = False
            if left == True and item.left == True:
                item.x -= 2
                direction= 1
            if right == True and item.right == True:
                item.x += 2
                direction = -1
            item.update(direction)
            for collideritem in colliderG:
                collideritem.x = item.x
                collideritem.y = item.y
                collideritem.update()
            # this is so that later on I can make it false if the octopus is colliding with an object
            # i do it so that the character doesn't walk through walls
            item.down = True
            item.left = True
            item.right = True

        for item in colliderG:
            for staticitem in staticG:
                if item.rect.colliderect(staticitem.rect):
                    if staticitem.type == "dirt" or staticitem.type == "grass":
                        if item.type == "down":
                            for octopus in octopusG:
                                octopus.fallspeed = 0
                                octopus.down = False
                        if item.type == "left":
                            for octopus in octopusG:
                                octopus.left = False
                        if item.type == "right":
                            for octopus in octopusG:
                                octopus.right = False
        
        backgroundG.draw(screen)
        staticG.draw(screen)
        octopusG.draw(screen)
        colliderG.draw(screen)
        pygame.display.update()
        clock.tick(60)

# This is the controller of the controllers
def main():
    # this sets up the screen
    screen = pygame.display.set_mode((640, 480), 0, 32)
    pygame.display.set_caption('Mr Octopus!!!')
    pygame.display.set_icon(loadIm("icon.png"))
    screen.fill(BLUE)
    Menu(screen)
    Game(screen)
    pygame.display.update()
    x = 255
    y = 255
    clock.tick()
    time.sleep(0.1)
    while True:
        screen.fill(BLUE)
        pygame.draw.circle(screen, BLACK, (int(x), int(y)), 20)
        pygame.display.update()
        clock.tick()
        x += (clock.get_time()) * 0.1
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
        time.sleep(random.random()/10)
